package com.example.lingobeemod;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class InternalDatabaseHandler extends SQLiteOpenHelper{

	// All Static variables
	// Database Version
	private static final int DATABASE_VERSION = 1;

	// Database Name
	private static final String DATABASE_NAME = "userDatabase";

	// User table name
	private static final String TABLE_USER = "user";
	
	// User Columns names
	private static final String KEY_USERID = "id_user";
	private static final String KEY_USERNAME = "username";
	private static final String KEY_MUG = "mug";
	private static final String KEY_ID_GROUP = "id_group";
	private static final String KEY_GROUPNAME = "groupname";
	private static final String KEY_GROUPCAT = "groupcat";
	private static final String KEY_GROUPLANG = "grouplang";
	private static final String KEY_POINTS = "userpoints";
	private static final String KEY_POSTPOINTS = "postpoints";
	private static final String KEY_FLAGPOINTS = "flagpoints";
	private static final String KEY_IMAGEPOINTS = "imagepoints";
	private static final String KEY_AUDIOPOINTS = "audiopoints";
	private static final String KEY_LINKPOINTS = "linkpoints";
	private static final String KEY_CATEGORYPOINTS = "categorypoints";
	private static final String KEY_LASTUPDATED = "lastupdated";

	// Word table name
	private static final String TABLE_WORD = "word";
	// Word Columns names
	private static final String KEY_ID_WORD = "id_word";
	private static final String KEY_WORD = "word";
	private static final String KEY_ID_USER = "id_user";
	private static final String KEY_CAT = "cat";
	private static final String KEY_LINK = "link";
	private static final String KEY_IMAGE = "image";
	private static final String KEY_AUDIO = "audio";
	private static final String KEY_FLAG = "flag";
	
	//Handle login
	private static final String TABLE_PASS = "pass";
	private static final String KEY_LOGINUSERNAME = "loginusername";
	private static final String KEY_LOGINPASS = "loginpassword";
	private static final String[] COLUMNS_PASS = { KEY_LOGINPASS, KEY_LOGINUSERNAME };

	private static final String[] COLUMNS = {
		KEY_USERID,
		KEY_MUG,
		KEY_USERNAME,
		KEY_ID_GROUP,
		KEY_GROUPNAME,
		KEY_GROUPCAT,
		KEY_GROUPLANG,
		KEY_POINTS, 
		KEY_POSTPOINTS, 
		KEY_FLAGPOINTS, 
		KEY_IMAGEPOINTS, 
		KEY_AUDIOPOINTS, 
		KEY_LINKPOINTS,
		KEY_CATEGORYPOINTS, 
		KEY_LASTUPDATED
	};

	private static final String[] COLUMNS_WORD = {
		KEY_ID_WORD,KEY_WORD,KEY_ID_USER,
		KEY_CAT,KEY_LINK,KEY_IMAGE,
		KEY_AUDIO,KEY_FLAG
	};


	public InternalDatabaseHandler(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		String CREATE_USER_TABLE = 
				"CREATE TABLE " + TABLE_USER + " ( "
						+ KEY_USERID + " STRING PRIMARY KEY, " 
						+ KEY_MUG + " TEXT, "
						+ KEY_USERNAME + " TEXT, "
						+ KEY_ID_GROUP + " TEXT, "
						+ KEY_GROUPNAME + " TEXT, "
						+ KEY_GROUPCAT + " TEXT, "
						+ KEY_GROUPLANG + " TEXT, "
						+ KEY_POINTS + " INTEGER, "
						+ KEY_POSTPOINTS + " INTEGER, "
						+ KEY_FLAGPOINTS + " INTEGER, "
						+ KEY_IMAGEPOINTS + " INTEGER, "
						+ KEY_AUDIOPOINTS + " INTEGER, "
						+ KEY_LINKPOINTS + " INTEGER, "
						+ KEY_CATEGORYPOINTS + " INTEGER, "
						+ KEY_LASTUPDATED + " TEXT "
						+ ")";

		String CREATE_WORD_TABLE = 
				"CREATE TABLE " + TABLE_WORD + " ( "
						+ KEY_ID_WORD + " STRING PRIMARY KEY, " 
						+ KEY_WORD + " TEXT, "
						+ KEY_ID_USER + " TEXT, "
						+ KEY_CAT + " TEXT, "
						+ KEY_FLAG + " TEXT, "
						+ KEY_LINK + " TEXT, "
						+ KEY_IMAGE + " TEXT, "
						+ KEY_AUDIO + " TEXT "
						+ ")";

		String CREATE_PASS_TABLE = 
				"CREATE TABLE " + TABLE_PASS + " ( "
						+ KEY_LOGINUSERNAME + " STRING PRIMARY KEY, " 
						+ KEY_LOGINPASS + " TEXT "
						+ ")";

		db.execSQL(CREATE_USER_TABLE);
		db.execSQL(CREATE_WORD_TABLE);
		db.execSQL(CREATE_PASS_TABLE);
	}

	public void addUser (User user){
		//for logging
		Log.d("addUser", user.toString()); 
		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. create ContentValues to add key "column"/value
		ContentValues values = new ContentValues();
		values.put(KEY_USERID, user.getID_USER()); 
		values.put(KEY_MUG, user.getMUG()); 
		values.put(KEY_USERNAME, user.getUSERNAME()); 
		values.put(KEY_ID_GROUP, user.getID_GROUP()); 
		values.put(KEY_GROUPNAME, user.getGROUPNAME()); 
		values.put(KEY_GROUPCAT, user.getGROUPCAT()); 
		values.put(KEY_GROUPLANG, user.getGROUPLANG()); 
		values.put(KEY_POINTS, user.getPOINTS()); 
		values.put(KEY_POSTPOINTS, user.getPOSTPOINTS()); 
		values.put(KEY_FLAGPOINTS, user.getFLAGPOINTS()); 
		values.put(KEY_IMAGEPOINTS, user.getIMAGEPOINTS()); 
		values.put(KEY_AUDIOPOINTS, user.getAUDIOPOINTS()); 
		values.put(KEY_LINKPOINTS, user.getLINKPOINTS()); 
		values.put(KEY_CATEGORYPOINTS, user.getCATEGORYPOINTS()); 
		values.put(KEY_LASTUPDATED, user.getLASTUPDATED());
		// 3. insert
		db.insert(TABLE_USER, // table
				null, //nullColumnHack
				values); // key/value -> keys = column names/ values = column values
		// 4. close
		db.close(); 
	}

	public void addPass (LoginCredencials lc){
		//for logging
		Log.d("Adding logincred: ", lc.toString()); 
		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. create ContentValues to add key "column"/value
		ContentValues values = new ContentValues();
		values.put(KEY_LOGINUSERNAME, lc.getLogin());
		values.put(KEY_LOGINPASS, lc.getPassword());
		// 3. insert
		db.insert(TABLE_PASS, // table
				null, //nullColumnHack
				values); // key/value -> keys = column names/ values = column values
		// 4. close
		db.close(); 
	}
	
	public void addWord (Word word){
		//for logging
		Log.d("addWord: ", word.toString()); 
		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. create ContentValues to add key "column"/value
		ContentValues values = new ContentValues();
		values.put(KEY_ID_WORD, word.getID_WORD()); 
		values.put(KEY_WORD, word.getWORD()); 
		values.put(KEY_ID_USER, word.getID_USER()); 
		values.put(KEY_CAT, word.getCAT()); 
		values.put(KEY_FLAG, word.getFLAG()); 
		values.put(KEY_IMAGE, word.getIMAGE()); 
		values.put(KEY_AUDIO, word.getAUDIO()); 
		values.put(KEY_LINK, word.getLINK()); 
		// 3. insert
		db.insert(TABLE_WORD, // table
				null, //nullColumnHack
				values); // key/value -> keys = column names/ values = column values
		// 4. close
		db.close(); 
	}

	// Deleting single book
	public void deleteUser(User user) {

		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. delete
		db.delete(TABLE_USER,
				KEY_USERID+" = ?",
				new String[] { String.valueOf(user.getID_USER()) });
		// 3. close
		db.close();
		Log.d("Delete user", user.toString());
	}

	public void deletePass() {

		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. delete
		db.delete(TABLE_PASS, null, null);
		// 3. close
		db.close();
		Log.d("Deleted logincredencials", "ALL IS GONE");
	}
	
	public void deleteAllWords() {

		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. delete
		db.delete(TABLE_WORD, null, null);
		// 3. close
		db.close();
		Log.d("Deleted all words", "ALL IS GONE");
	}

	public void deleteWord(Word word) {

		// 1. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		// 2. delete
		db.delete(TABLE_WORD,
				KEY_ID_WORD+" = ?",
				new String[] { String.valueOf(word.getID_WORD()) });
		// 3. close
		db.close();
		Log.d("Delete word", word.toString());
	}

	public User getUser(String userID){	 
		// 1. get reference to readable DB
		SQLiteDatabase db = this.getReadableDatabase(); 
		// 2. build query
		Cursor cursor = 
				db.query(TABLE_USER, // a. table
						COLUMNS, // b. column names
						" id_user = ?", // c. selections 
						new String[] { String.valueOf(userID) }, // d. selections args
						null, // e. group by
						null, // f. having
						null, // g. order by
						null); // h. limit

		// 3. if we got results get the first one
		if (cursor != null)
			cursor.moveToFirst(); 

		// 4. build book object
		User user = new User();
		user.setID_USER(cursor.getString(cursor.getColumnIndex(KEY_ID_USER)));
		user.setMUG(cursor.getString(cursor.getColumnIndex(KEY_MUG)));
		user.setUSERNAME(cursor.getString(cursor.getColumnIndex(KEY_USERNAME)));
		user.setID_GROUP(cursor.getString(cursor.getColumnIndex(KEY_ID_GROUP)));
		user.setGROUPNAME(cursor.getString(cursor.getColumnIndex(KEY_GROUPNAME)));
		user.setGROUPCAT(cursor.getString(cursor.getColumnIndex(KEY_GROUPCAT)));
		user.setGROUPLANG(cursor.getString(cursor.getColumnIndex(KEY_GROUPLANG)));
		user.setPOINTS(cursor.getString(cursor.getColumnIndex(KEY_POINTS)));
		user.setPOSTPOINTS(cursor.getString(cursor.getColumnIndex(KEY_POSTPOINTS)));
		user.setFLAGPOINTS(cursor.getString(cursor.getColumnIndex(KEY_FLAGPOINTS)));
		user.setIMAGEPOINTS(cursor.getString(cursor.getColumnIndex(KEY_IMAGEPOINTS)));
		user.setAUDIOPOINTS(cursor.getString(cursor.getColumnIndex(KEY_AUDIOPOINTS)));
		user.setLINKPOINTS(cursor.getString(cursor.getColumnIndex(KEY_LINKPOINTS)));
		user.setCATEGORYPOINTS(cursor.getString(cursor.getColumnIndex(KEY_CATEGORYPOINTS)));
		user.setLASTUPDATED(cursor.getString(cursor.getColumnIndex(KEY_LASTUPDATED)));

		cursor.close();
		return user;
	}

	public List<String> getAllFlagsByUser(String userID){
		List<String> list = new LinkedList<String>();
		String query = "SELECT "+ KEY_FLAG + " FROM " + TABLE_WORD + " WHERE " + KEY_ID_USER + "=" + userID;
		SQLiteDatabase db = this.getWritableDatabase();
		String s = null;
		Cursor cursor = db.rawQuery(query, null);

		if (cursor.moveToFirst()) {
			do {
				s = new String();
				s = cursor.getString(0);
				list.add(s);
			} while (cursor.moveToNext());
		}
		cursor.close();    
		return list;
	}

	public List<String> getAllAudioByUser(String userID){
		List<String> list = new LinkedList<String>();
		String query = "SELECT "+ KEY_AUDIO + " FROM " + TABLE_WORD + " WHERE " + KEY_ID_USER + "=" + userID;
		SQLiteDatabase db = this.getWritableDatabase();
		String s = null;
		Cursor cursor = db.rawQuery(query, null);

		if (cursor.moveToFirst()) {
			do {
				s = new String();
				s = cursor.getString(0);
				list.add(s);
			} while (cursor.moveToNext());
		}
		cursor.close();    
		return list;
	}	

	public List<String> getAllImagesByUser(String userID){
		List<String> list = new LinkedList<String>();
		String query = "SELECT "+ KEY_IMAGE + " FROM " + TABLE_WORD + " WHERE " + KEY_ID_USER + "=" + userID;
		SQLiteDatabase db = this.getWritableDatabase();
		String s = null;
		Cursor cursor = db.rawQuery(query, null);

		if (cursor.moveToFirst()) {
			do {
				s = new String();
				s = cursor.getString(0);
				list.add(s);
			} while (cursor.moveToNext());
		}
		cursor.close();    
		return list;
	}

	public List<String> getAllLinksByUser(String userID){
		List<String> linkString = new LinkedList<String>();
		String query = "SELECT "+ KEY_LINK + " FROM " + TABLE_WORD + " WHERE " + KEY_ID_USER + "=" + userID;
		SQLiteDatabase db = this.getWritableDatabase();
		String link = null;
		Cursor cursor = db.rawQuery(query, null);

		if (cursor.moveToFirst()) {
			do {
				link = new String();
				link = cursor.getString(0);
				linkString.add(link);
			} while (cursor.moveToNext());
		}
		cursor.close();    
		return linkString;
	}

	public List<String> getAllCatsByUser(String userId){
		List<String> catList = new LinkedList<String>();
		String query = "SELECT "+ KEY_CAT + " FROM " + TABLE_WORD + " WHERE " + KEY_ID_USER + "=" + userId;

		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);

		String cat = null;

		if (cursor.moveToFirst()) {
			do {
				cat = new String();
				cat = cursor.getString(0);
				catList.add(cat);
			} while (cursor.moveToNext());
		}
		cursor.close();
		return catList;
	}

	public LoginCredencials getLoginCredencials(String username) {
		// 1. get reference to readable DB
		SQLiteDatabase db = this.getReadableDatabase(); 
		// 2. build query
		Cursor cursor = 
				db.query(TABLE_PASS, // a. table
						COLUMNS_PASS, // b. column names
						" loginusername = ?", // c. selections 
						new String[] { String.valueOf(username) }, // d. selections args
						null, // e. group by
						null, // f. having
						null, // g. order by
						null); // h. limit

		if (cursor != null)
			cursor.moveToFirst(); 
		LoginCredencials lc = new LoginCredencials();
		lc.setLogin(cursor.getString(0));
		lc.setPassword(cursor.getString(1));
		cursor.close();
		return lc;
	}

	public Word getWord(String wordId){	 
		// 1. get reference to readable DB
		SQLiteDatabase db = this.getReadableDatabase(); 
		// 2. build query
		Cursor cursor = 
				db.query(TABLE_WORD, // a. table
						COLUMNS_WORD, // b. column names
						" id_user = ?", // c. selections 
						new String[] { String.valueOf(wordId) }, // d. selections args
						null, // e. group by
						null, // f. having
						null, // g. order by
						null); // h. limit

		// 3. if we got results get the first one
		if (cursor != null)
			cursor.moveToFirst(); 
		// 4. build book object
		Word word = new Word();
		word.setID_WORD(cursor.getString(0));
		word.setWORD(cursor.getString(1));
		word.setID_USER(cursor.getString(2));
		word.setCAT(cursor.getString(3));
		word.setFLAG(cursor.getString(4));
		word.setLINK(cursor.getString(5));
		word.setIMAGE(cursor.getString(6));
		word.setAUDIO(cursor.getString(6));
		//log 
		Log.d("Get word ("+wordId+")", word.toString());	 
		// 5. return book
		cursor.close();
		return word;
	}

	public List<Word> getAllWordsByUser(String userId) {
		List<Word> wordList = new LinkedList<Word>();

		// 1. build the query
		String query = "SELECT * FROM " + TABLE_WORD + " WHERE " + KEY_ID_USER + "=" + userId;

		// 2. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);

		// 3. go over each row, build book and add it to list
		Word word = null;
		if (cursor.moveToFirst()) {
			do {
				word = new Word();
				word.setID_WORD(cursor.getString(0));

				// Add book to books
				wordList.add(word);
			} while (cursor.moveToNext());
		}

		Log.d("GetAllWords()", wordList.toString());

		// return books
		cursor.close();
		return wordList;
	}

	public List<Word> getAllWords() {
		List<Word> wordList = new LinkedList<Word>();

		// 1. build the query
		String query = "SELECT  * FROM " + TABLE_WORD + " ORDER BY " + KEY_ID_WORD + " DESC";
		
		// 2. get reference to writable DB
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);

		// 3. go over each row, build book and add it to list
		Word word = null;
		if (cursor.moveToFirst()) {
			do {
				word = new Word();
				word.setID_WORD(cursor.getString(0));

				// Add book to books
				wordList.add(word);
			} while (cursor.moveToNext());
		}

		Log.d("GetAllWords()", wordList.toString());

		// return books
		cursor.close();
		return wordList;
	}

	public List<User> getTopUsers() {
		List<User> userList = new LinkedList<User>();

		// 1. build the query
		String query = "SELECT  * FROM " + TABLE_USER +" ORDER BY " + KEY_POINTS + " DESC";
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);
		User user = null;
		if (cursor.moveToFirst()) {
			do {
				user = new User();
				user.setID_USER(cursor.getString(0));

				// Add book to books
				userList.add(user);
			} while (cursor.moveToNext());
		}

		Log.d("GettingTopUser()", userList.toString());

		// return books
		cursor.close();
		return userList;
	}

	public List<LoginCredencials> getRememberedLoginCredencials(){
		List<LoginCredencials> creList = new LinkedList<LoginCredencials>();

		String query = "SELECT * FROM " + TABLE_PASS;
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);
		LoginCredencials creds = null;
		if (cursor.moveToFirst()) {
			do {
				creds = new LoginCredencials();
				creds.setLogin(cursor.getString(0));
				creds.setPassword(cursor.getString(1));

				// Add book to books
				creList.add(creds);
			} while (cursor.moveToNext());
		}
		cursor.close();
		return creList;
	}

	public List<User> getAllUsers() {
		List<User> userList = new LinkedList<User>();

		// 1. build the query
		String query = "SELECT * FROM " + TABLE_USER;
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(query, null);
		User user = null;
		if (cursor.moveToFirst()) {
			do {
				user = new User();
				user.setID_USER(cursor.getString(0));

				// Add book to books
				userList.add(user);
			} while (cursor.moveToNext());
		}

		Log.d("GetAllUsers()", userList.toString());

		// return books
		cursor.close();
		return userList;
	}

	public int updateUser(User user) {
		Log.i("Updating user", "Started");
		SQLiteDatabase db = this.getWritableDatabase();
		ContentValues values = new ContentValues();
		values.put(KEY_LASTUPDATED, user.getLASTUPDATED());
		values.put(KEY_USERNAME, user.getUSERNAME());
		values.put(KEY_POINTS, user.getPOINTS()); 
		values.put(KEY_POSTPOINTS, user.getPOSTPOINTS()); 
		values.put(KEY_FLAGPOINTS, user.getFLAGPOINTS()); 
		values.put(KEY_IMAGEPOINTS, user.getIMAGEPOINTS()); 
		values.put(KEY_AUDIOPOINTS, user.getAUDIOPOINTS()); 
		values.put(KEY_LINKPOINTS, user.getLINKPOINTS());
		values.put(KEY_CATEGORYPOINTS, user.getCATEGORYPOINTS()); 

		// 3. updating row
		int i = db.update(TABLE_USER, //table
				values, // column/value
				KEY_USERID+" = ?", // selections
				new String[] { String.valueOf(user.getID_USER()) }); //selection args

		db.close(); 
		Log.i("Updating user", "Finished");
		return i;    
	}

	public int updateWord(Word word) {
		Log.i("Updating word: " , " Started");
		SQLiteDatabase db = this.getWritableDatabase();    	
		ContentValues values = new ContentValues();
		values.put("id_user", word.getID_USER()); 
		values.put("cat", word.getCAT()); 
		values.put("link", word.getLINK()); 
		values.put("image", word.getIMAGE()); 
		values.put("audio", word.getAUDIO()); 
		values.put("flag", word.getFLAG()); 

		int i = db.update(TABLE_WORD, //table
				values, // column/value
				KEY_ID_WORD+" = ?", // selections
				new String[] { String.valueOf(word.getID_WORD()) }); //selection args

		db.close(); 
		Log.i("Updating word: " , " Finished");	 
		return i; 
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// Drop older table if existed
		db.execSQL("DROP TABLE IF EXISTS " + TABLE_USER);
		// Create tables again
		onCreate(db);
	}
}
